/*
 * Project name:
     RTC_Read (Demonstration on Working with the RTC Module)
 * Copyright:
     (c) MikroElektronika, 2006.
 * Description:
      This project is simple demonstration how to Read date and time from
      PCF8583 RTC (real-time clock). The code can be used with any MCU that has
      MSSP module at PORTC.
      The offset from Y2000 is stored in the PCF8583 RAM at address 0x10.
      The time format is 24h and weekdays are not used.
 * Test configuration:
     MCU:             PIC16F887
     Dev.Board:       EasyPIC5
     Oscillator:      HS, 8.000 MHz
     Ext. Modules:    RTC on PORTC
     SW:              mikroC Pro v6.0.0.0
 * NOTES:
     - Be sure to switch the portC LEDs OFF
     - For proper I2C communication, PORTC must be in the pull-up mode
     - In order to use the example, address pin A0 of PCF8583 must be set to 0V.
       (on mikroElektronika's RTC module this is done by default)
 */
struct TTime
{
  char year, month, day, hours, minutes, seconds, mseconds;
} TimeRW;

#define _USE_PCF8583_SOFT_ 0
#define _USE_PCF8583_HARD_ 1
#include ".././lib/Lib_RTC8583_MikroC.h"

/**************************************************************************************************/
void UART1_jump_a_line()
{
 UART1_Write(10);
 UART1_Write(13);
}

char const weekday_names[7][10] =
{
 {"Sunday"},
 {"Monday"},
 {"Tuesday"},
 {"Wednesday"},
 {"Thursday"},
 {"Friday"},
 {"Saturday"}
};

void Display_Time()
{
 char txt[16], tmp;
 Lcd_Cmd(_LCD_CLEAR);               // Clear display
 Lcd_Cmd(_LCD_CURSOR_OFF);          // Cursor off
 //lcd_putc('\f');
 Lcd_Cmd(_LCD_FIRST_ROW);
 UART1_Write_Text("Time: ");

   // output values to LCD display
   txt[0] = (_RTC_day_ / 10) + 48;
   txt[1] = (_RTC_day_ % 10) + 48;
   txt[2] = '/';
   txt[3] = (_RTC_month_ / 10) + 48;
   txt[4] = (_RTC_month_ % 10) + 48;
   txt[5] = '/';
   txt[6] = '2';
   txt[7] = (_RTC_year_ / 100) + 48;
   txt[8] = ((_RTC_year_ % 100) / 10) + 48;
   txt[9] = (_RTC_year_ % 10)  + 48;
   txt[10] = 0;  // null to terminate the string
   //Lcd_Out(1,7,txt);
   for(tmp=0;tmp<10;tmp++)
   {
       UART1_Write(txt[tmp]);
       Lcd_Chr_CP(txt[tmp]);
   }

   UART1_Write_Text(" - ");
   Lcd_Cmd(_LCD_SECOND_ROW);

   txt[0] = (_RTC_hours_ / 10) + 48;
   txt[1] = (_RTC_hours_ % 10) + 48;
   txt[2] = ':';
   txt[3] = (_RTC_minutes_ / 10) + 48;
   txt[4] = (_RTC_minutes_ % 10) + 48;
   txt[5] = ':';
   txt[6] = (_RTC_seconds_ / 10) + 48;
   txt[7] = (_RTC_seconds_ % 10) + 48;
   txt[8] = '.';
   txt[9] = (_RTC_mseconds_ / 10) + 48;
   txt[10] = (_RTC_mseconds_ % 10) + 48;
   txt[11] = 's';
   txt[12] = 0;  // null to terminate the string
   //Lcd_Out(2,7,txt);
   for(tmp=0;tmp<12;tmp++)
   {
    UART1_Write(txt[tmp]);
      Lcd_Chr_CP(txt[tmp]);
   }
   UART1_jump_a_line();
}

// LCD module connections
sbit LCD_RS at RD5_bit;
sbit LCD_EN at RD7_bit;

sbit LCD_D4 at RD0_bit;
sbit LCD_D5 at RD2_bit;
sbit LCD_D6 at RD4_bit;
sbit LCD_D7 at RD6_bit;

sbit LCD_RS_Direction at TRISD5_bit;  // bleu
sbit LCD_EN_Direction at TRISD7_bit;  // orange

sbit LCD_D4_Direction at TRISD0_bit;  // vert
sbit LCD_D5_Direction at TRISD2_bit;  // jaune
sbit LCD_D6_Direction at TRISD4_bit;  // gris
sbit LCD_D7_Direction at TRISD6_bit;  // rose
// RW = GND
// End LCD module connections

void MainInit()
{
  ANSEL = 0;
  ANSELH = 0;
  
  UART1_Init(9600);               // Initialize UART module at 9600 bps
  Lcd_Init();                        // Initialize LCD
  I2C1_Init(10000);      // initialize full master mode I2C

  UART1_Write_Text("Build : " __TIME__);
  UART1_Write_Text(", " __DATE__);
  UART1_jump_a_line();
  Delay_ms(1000);                  // Wait for to stabilize
  
  Lcd_Cmd(_LCD_CLEAR);
  Lcd_Cmd(_LCD_CURSOR_OFF);
  Lcd_Cmd(_LCD_FIRST_ROW);
  Lcd_Out(1,1,"Date:   /  /");
  Lcd_Out(2,1,"Time:   :  :");

  Clock_Out_RTC_1Hz();
  
  TimeRW.year    = 18;    // 2018 (offset from 2000)
  TimeRW.month   = 12;
  TimeRW.day     = 31;
  TimeRW.hours   = 23;
  TimeRW.minutes = 59;
  TimeRW.seconds = 40;
  TimeRW.mseconds = 0;
  
  WriteTime(TimeRW.year, TimeRW.month,TimeRW.day, TimeRW.hours, TimeRW.minutes, TimeRW.seconds, TimeRW.mseconds);
}



void main()
{
   MainInit();

   while(1)
   {
     ReadTime();
     Display_Time();
     Delay_ms(500);
   }
}