/*

Library for PCF8583 version 2.0.0.2 (Hardware & Soft)


      The offset from Y2000 is stored in the PCF8583 RAM at address 0x10.
      The time format is 24h and weekdays are not used.
      
  - unsigned char RTC_Read(unsigned char addr)
  - void RTC_Write(unsigned char addr, unsigned char value
  - void Init_TimeRW_Default_Value()
  - void WriteTime()
  - void ReadTime()
  
  P.S :
  - void RTC_Clock_Out_1Hz() ne fonctionne pas dans la simulation Proteus
*/

// RTC Definitions (PCF8583)
#define adresse_PCF8583_Writing   0xA0    // address PCF8583 for Writing R/W=0
#define adresse_PCF8583_Reading   0xA1    // address PCF8583 for Reading R/W=1

// Register addresses
#define PCF8583_CTRL_STATUS_REG   0x00
#define PCF8583_100S_REG          0x01
#define PCF8583_SECONDS_REG       0x02
#define PCF8583_MINUTES_REG       0x03
#define PCF8583_HOURS_REG         0x04
#define PCF8583_DATE_REG          0x05
#define PCF8583_MONTHS_REG        0x06
#define PCF8583_TIMER_REG         0x07
#define PCF8583_ALARM_CONTROL_REG 0x08
#define PCF8583_ALARM_100S_REG    0x09
#define PCF8583_ALARM_SECS_REG    0x0A
#define PCF8583_ALARM_MINS_REG    0x0B
#define PCF8583_ALARM_HOURS_REG   0x0C
#define PCF8583_ALARM_DATE_REG    0x0D
#define PCF8583_ALARM_MONTHS_REG  0x0E
#define PCF8583_ALARM_TIMER_REG   0x0F
// Use the first NVRAM address for the year byte.
#define PCF8583_YEAR_REG          0x10
// Commands for the Control/Status register.
#define PCF8583_START_COUNTING    0x00
#define PCF8583_STOP_COUNTING     0x80

#if  _USE_PCF8583_SOFT_
#define _I2C_Init()              Soft_I2C_Init()
#define _I2C_Start()             Soft_I2C_Start()
#define _I2C_Repeated_Start()    Soft_I2C_Start()
#define _I2C_Wr(x)               Soft_I2C_Write(x)
#define _I2C_Rd(x)               Soft_I2C_Read(x)
#define _I2C_Stop()              Soft_I2C_Stop()
#endif

#if  _USE_PCF8583_HARD_
#define _I2C_Init(x)             I2C1_Init(x)
#define _I2C_Start()             I2C1_Start()
#define _I2C_Repeated_Start()    I2C1_Repeated_Start()
#define _I2C_Wr(x)               I2C1_Wr(x)
#define _I2C_Rd(x)               I2C1_Rd(x)
#define _I2C_Stop()              I2C1_Stop()
#endif

char _RTC_year_, _RTC_month_, _RTC_day_, _RTC_hours_, _RTC_minutes_, _RTC_seconds_, _RTC_mseconds_;

/**************************************************************************************************
* Read data from RTC PCF8583
* input : address of RTC register
* output: value of of RTC register
**************************************************************************************************/
unsigned char RTC_Read_PCF8583(unsigned char addr)
{
  unsigned char value;

  _I2C_Start();                   // Issue start signal
  _I2C_Wr(adresse_PCF8583_Writing);              // Address PCF8583, see PCF8583 datasheet
  _I2C_Wr(addr);                  // Start from address 2
  _I2C_Start();                   // Issue repeated start signal
  _I2C_Wr(adresse_PCF8583_Writing);              // Address PCF8583 for reading R/W=1

  value = _I2C_Rd(0);             // Read seconds byte
  _I2C_Stop();                    // Issue stop signal

  return value;
}

/**************************************************************************************************
* Write data from RTC PCF8583
* input : address of RTC register, value of of RTC register
**************************************************************************************************/
void RTC_Write_PCF8583(unsigned char addr, unsigned char value)
{
  _I2C_Start();         // Issue start signal
  _I2C_Wr(adresse_PCF8583_Writing);    // Address PCF8583
  _I2C_Wr(addr);        // Start from address
  _I2C_Wr(value);       // Write value to RTC register
  _I2C_Stop();          // Issue stop signal
}

void WriteTime(char year, char month, char day, char hours, char minutes, char seconds, char mseconds)
{
   _RTC_year_ = year, _RTC_month_ = month, _RTC_day_ = day;
   _RTC_hours_ = hours, _RTC_minutes_ = minutes, _RTC_seconds_ = seconds, _RTC_mseconds_ = mseconds;
   
   _I2C_Start();            // issue start signal
   _I2C_Wr(adresse_PCF8583_Writing);          // address PCF8583
   _I2C_Wr(PCF8583_CTRL_STATUS_REG);            // start from word at address 0 (configuration word)
   _I2C_Wr(PCF8583_STOP_COUNTING);          // write 0x80 to config. (pause counter...)
   _I2C_Wr(mseconds);            // write 0 to cents word
   _I2C_Wr(((seconds/10)<<4) + (seconds%10)); // write seconds word
   _I2C_Wr(((minutes/10)<<4) + (minutes%10)); // write minutes word
   _I2C_Wr(((hours/10)<<4) + (hours%10)); // write hours word
   _I2C_Wr(((year%4)<<6) + ((day/10)<<4) +(day%10));
   _I2C_Wr(((month/10)<<4) + (month%10));  // write weekday/month
   _I2C_Stop();             // issue stop signal

   RTC_Write_PCF8583(PCF8583_YEAR_REG, year);
   /*_I2C_Start();            // issue start signal
   _I2C_Wr(adresse_PCF8583_Writing);          // address PCF8583
   _I2C_Wr(PCF8583_YEAR_REG);          // start from word at address 0x10
   _I2C_Wr(year);         // write year to RAM
   _I2C_Stop();             // issue stop signal
    */
   RTC_Write_PCF8583(PCF8583_CTRL_STATUS_REG, PCF8583_START_COUNTING);
   /*
   _I2C_Start();            // issue start signal
   _I2C_Wr(adresse_PCF8583_Writing);          // address PCF8583
   _I2C_Wr(PCF8583_CTRL_STATUS_REG);            // start from word at address 0
   _I2C_Wr(PCF8583_START_COUNTING);            // write 0 to config word (enable counting)
   _I2C_Stop();             // issue stop signal
   */
}

void ReadTime()
{
 char updateYear, yearmod4, byteRead;
 char year, month, day, hours, minutes, seconds, mseconds;
 
 updateYear = 0;

   _I2C_Start();                // issue start signal
   _I2C_Wr(adresse_PCF8583_Writing);               // address PCF8583
   _I2C_Wr(PCF8583_100S_REG);                  // first word address
   _I2C_Repeated_Start();       // issue repeated start signal
   _I2C_Wr(adresse_PCF8583_Reading);               // address PCF8583 for reading R/W=1

   byteRead = _I2C_Rd(1);       // read seconds byte
   mseconds = (byteRead >> 4)*10 + (byteRead & 0x0F); // transform ms

   byteRead = _I2C_Rd(1);       // read seconds byte
   seconds = (byteRead >> 4)*10 + (byteRead & 0x0F); // transform seconds

   byteRead = _I2C_Rd(1);     // read minutes byte
   minutes = (byteRead >> 4)*10 + (byteRead & 0x0F); // transform minutes

   byteRead = _I2C_Rd(1);     // read hours byte
   hours = (byteRead >> 4)*10 + (byteRead & 0x0F); // transform hours

   byteRead = _I2C_Rd(1);     // read year/day byte
   day = ((byteRead & 0b00110000) >> 4)*10 + (byteRead & 0x0F); // transform day
   yearmod4 = (byteRead & 0b11000000) >> 6;   // get year mod 4 from RTC

   byteRead = _I2C_Rd(0);     // read weekday/month byte
   month = ((byteRead & 0b00010000) >> 4)*10 + (byteRead & 0x0F); // transform month

   _I2C_Stop();

   _I2C_Start();                 // issue start signal
   _I2C_Wr(adresse_PCF8583_Writing);               // address PCF8583
   _I2C_Wr(PCF8583_YEAR_REG);               // first word address
   _I2C_Repeated_Start();        // issue repeated start signal
   _I2C_Wr(adresse_PCF8583_Reading);               // address PCF8583 for reading R/W=1

   byteRead = _I2C_Rd(0);     // read year
   if(yearmod4 != byteRead % 4 )
   { // check if year is incremented in RTC
       byteRead++;  // in this case the new value should be written to RTC RAM at address 16 (0x10)
       updateYear = 1;
   }
   year = byteRead;

   _I2C_Stop();

   if(updateYear > 0)
   {
      /*_I2C_Start();            // issue start signal
      _I2C_Wr(adresse_PCF8583_Writing);          // address PCF8583
      _I2C_Wr(PCF8583_YEAR_REG);          // start from word at address 0x10
      _I2C_Wr(year);         // write year to RAM
      _I2C_Stop();             // issue stop signal
      */
      RTC_Write_PCF8583(PCF8583_YEAR_REG, year);
    }
    
    _RTC_year_ = year, _RTC_month_ = month, _RTC_day_ = day;
   _RTC_hours_ = hours, _RTC_minutes_ = minutes, _RTC_seconds_ = seconds, _RTC_mseconds_ = mseconds;
}
/**************************************************************************************************
* Write to RTC PCF8583 for Clock Out RTC=1Hz
**************************************************************************************************/
void Clock_Out_RTC_1Hz()
{
  /*
 _I2C_Start();
 _I2C_Wr(adresse_PCF8583_Writing);
 _I2C_Wr(PCF8583_ALARM_DATE_REG); //0x0D);
 _I2C_Wr(0x83);
 _I2C_Stop();
 */
 //RTC_Write_PCF8583(PCF8583_ALARM_DATE_REG, 0x83);
 RTC_Write_PCF8583(0,0b00000001); //SQWE output at 1 Hz
}